import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import java.util.ArrayList;

public class MessreiheGesamtsummeTest{
    private Messreihe m33244;
    private Messreihe m27;
    private Messreihe m2344;
    private Messreihe m3;
    private Messreihe leer;
    private Messreihe mitNullListe;
  private ArrayList<Integer> genutzt;

  private void merkeGenutzteWerte(Messreihe m){
    if (m.getMesswerte() == null){
      return;
    }
    for (int w: m.getMesswerte()) { // in Messreihe keine null-Werte
      this.genutzt.add(w);
    }
  }
  
  private boolean messwerteUnveraendert(Messreihe m){
    if (m.getMesswerte() == null){
      return this.genutzt.size() == 0;
    }
    for(int i = 0; i < this.genutzt.size(); i++) {
      if (this.genutzt.get(i) != m.getMesswerte().get(i)){
        return false;
      }
    }
    return m.getMesswerte().size() == this.genutzt.size();
  }
  
  @BeforeEach
  public void setUp() {
    this.genutzt = new ArrayList<Integer>();
        this.m33244 = new Messreihe("Ort1");
        this.m33244.hinzufuegen(3);
        this.m33244.hinzufuegen(3);
        this.m33244.hinzufuegen(2);
        this.m33244.hinzufuegen(4);
        this.m33244.hinzufuegen(4);
        this.m27 = new Messreihe("Ort2");
        this.m27.hinzufuegen(2);
        this.m27.hinzufuegen(7);
        this.m2344 = new Messreihe("Ort3");
        this.m2344.hinzufuegen(2);
        this.m2344.hinzufuegen(3);
        this.m2344.hinzufuegen(4);
        this.m2344.hinzufuegen(4);   
        this.leer = new Messreihe("Ort4");
        this.mitNullListe = new Messreihe("Ort5");
        this.mitNullListe.setMesswerte(null);
        this.m3 = new Messreihe("Ort6");
        this.m3.hinzufuegen(3);
    }

    @Test
    public void test01(){
        this.merkeGenutzteWerte(this.m33244);
        Assertions.assertEquals(16
                , this.m33244.gesamtsumme()
                , "Fehler fuer " + this.m33244 );
        Assertions.assertTrue(this.messwerteUnveraendert(this.m33244)
          , "urspruengliche Werte " + this.genutzt 
            + " sollten nicht geaendert werden:" + this.m33244);        
    }   
    
    @Test
    public void test02(){
        this.merkeGenutzteWerte(this.m27);
        Assertions.assertEquals(9
                , this.m27.gesamtsumme()
                , "Fehler fuer " + this.m27);
        Assertions.assertTrue(this.messwerteUnveraendert(this.m27)
          , "urspruengliche Werte " + this.genutzt 
            + " sollten nicht geaendert werden:" + this.m27);        
    }
    
    @Test
    public void test03(){
        this.merkeGenutzteWerte(this.m2344);
        Assertions.assertEquals(13
                , this.m2344.gesamtsumme()
                , "Fehler fuer " + this.m2344);
        Assertions.assertTrue(this.messwerteUnveraendert(this.m2344)
          , "urspruengliche Werte " + this.genutzt 
            + " sollten nicht geaendert werden:" + this.m2344);
    }
    
    @Test
    public void test04(){
        this.merkeGenutzteWerte(this.leer);
        Assertions.assertEquals(0
                , this.leer.gesamtsumme()
                , "Fehler fuer " + this.leer);
        Assertions.assertTrue(this.messwerteUnveraendert(this.leer)
          , "urspruengliche Werte " + this.genutzt 
            + " sollten nicht geaendert werden:" + this.leer);        
    }
    
    @Test
    public void test05(){
        this.merkeGenutzteWerte(this.mitNullListe);
        Assertions.assertEquals(0
                , this.mitNullListe.gesamtsumme()
                , "Fehler fuer " + this.mitNullListe);
        Assertions.assertTrue(this.messwerteUnveraendert(this.mitNullListe)
          , "urspruengliche Werte " + this.genutzt 
            + " sollten nicht geaendert werden:" + this.mitNullListe);            
    }

    @Test
    public void test06(){
        this.merkeGenutzteWerte(this.m3);
        Assertions.assertEquals(3
                , this.m3.gesamtsumme()
                , "Fehler fuer " + this.m3);
        Assertions.assertTrue(this.messwerteUnveraendert(this.m3)
          , "urspruengliche Werte " + this.genutzt 
            + " sollten nicht geaendert werden:" + this.m3);
    }
}
